@@ -1,7 +1,7 @@
 /**************************************************************************
  *   prototypes.h  --  This file is part of GNU nano.                     *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -36,7 +36,7 @@ extern bool bracketed_paste;
 
 extern bool we_are_running;
 extern bool more_than_one;
-
+extern bool report_size;
 extern bool ran_a_tool;
 
 extern bool inhelp;
@@ -53,6 +53,7 @@ extern message_type lastmessage;
 extern linestruct *pletion_line;
 
 extern bool also_the_last;
+extern bool hide_cursor;
 
 extern char *answer;
 
@@ -186,15 +187,10 @@ extern size_t light_to_col;
 
 typedef void (*functionptrtype)(void);
 
-/* Most functions in browser.c. */
+/* The two needed functions from browser.c. */
 #ifdef ENABLE_BROWSER
-char *browse_in(const char *inpath);
-void read_the_list(const char *path, DIR *dir);
 void browser_refresh(void);
-void browser_select_dirname(const char *needle);
-void do_filesearch(bool forwards);
-void do_fileresearch(bool forwards);
-char *strip_last_component(const char *path);
+char *browse_in(const char *inpath);
 #endif
 
 /* Most functions in chars.c. */
@@ -209,6 +205,7 @@ bool is_word_char(const char *c, bool allow_punct);
 char control_mbrep(const char *c, bool isdata);
 #ifdef ENABLE_UTF8
 int mbwidth(const char *c);
+bool is_zerowidth(const char *ch);
 char *make_mbchar(long code, int *length);
 #endif
 int char_length(const char *pointer);
@@ -242,7 +239,6 @@ bool is_valid_unicode(wchar_t wc);
 void set_interface_colorpairs(void);
 void prepare_palette(void);
 void find_and_prime_applicable_syntax(void);
-void set_up_multicache(linestruct *line);
 void check_the_multis(linestruct *line);
 void precalc_multicolorinfo(void);
 #endif
@@ -395,9 +391,6 @@ void free_lines(linestruct *src);
 void renumber_from(linestruct *line);
 void print_view_warning(void);
 bool in_restricted_mode(void);
-#ifndef ENABLE_HELP
-void say_there_is_no_help(void);
-#endif
 void finish(void);
 void close_and_go(void);
 void do_exit(void);
@@ -405,18 +398,20 @@ void die(const char *msg, ...);
 void window_init(void);
 void install_handler_for_Ctrl_C(void);
 void restore_handler_for_Ctrl_C(void);
+#ifndef NANO_TINY
 void reconnect_and_store_state(void);
-RETSIGTYPE handle_hupterm(int signal);
-#ifndef DEBUG
-RETSIGTYPE handle_crash(int signal);
 #endif
-RETSIGTYPE do_suspend(int signal);
-RETSIGTYPE do_continue(int signal);
+void handle_hupterm(int signal);
+#ifndef DEBUG
+void handle_crash(int signal);
+#endif
+void do_suspend(int signal);
+void do_continue(int signal);
 #if !defined(NANO_TINY) || defined(ENABLE_SPELLER) || defined(ENABLE_COLOR)
 void block_sigwinch(bool blockit);
 #endif
 #ifndef NANO_TINY
-RETSIGTYPE handle_sigwinch(int signal);
+void handle_sigwinch(int signal);
 void compute_the_extra_rows_per_line_from(linestruct *fromline);
 void regenerate_screen(void);
 void do_toggle(int flag);
@@ -595,9 +590,11 @@ void blank_statusbar(void);
 void wipe_statusbar(void);
 void blank_bottombars(void);
 void check_statusblank(void);
+void set_blankdelay_to_one(void);
 char *display_string(const char *buf, size_t column, size_t span,
 						bool isdata, bool isprompt);
 void titlebar(const char *path);
+void minibar(void);
 void statusline(message_type importance, const char *msg, ...);
 void statusbar(const char *msg);
 void warn_and_briefly_pause(const char *msg);
